/*
 *	Zones a la Amos
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <string.h>

#include <unistd.h>

#include "hr_defines.h"
#include "hr_general.h"
#include "hr_windows.h"
#include "hr_screens.h"

struct hr_zone{
	int flags;
	struct hr_zone *next;
	struct hr_screen *parent_screen;
	struct hr_window *parent_window;
	int x1;
	int y1;
	int x2;
	int y2;
};


/* Creates a zone. Returns pointer to the new zone. */
struct hr_zone *hr_create_zone(int x1,int y1,int x2,int y2,int flags)
{
	struct hr_zone *p=NULL;
	if(hr.active_screen!=NULL){
		if((p=malloc(sizeof(struct hr_zone)))){
			p->next=hr.firstzone;
			hr.firstzone=p;
			p->flags=flags;
			p->x1=MIN(x1,x2);
			p->y1=MIN(y1,y2);
			p->x2=MAX(x1,x2);
			p->y2=MAX(y1,y2);
			p->parent_screen=hr.active_screen;
			p->parent_window=hr.active_screen->active_window;
		}
	}
	return p;
}

/* Returns pointer to first zone under coordinates x,y
 * or NULL if no button is there.*/
struct hr_zone *hr_get_zone(struct hr_zone *firstzone,int x,int y,int flags)
{
	struct hr_zone *z;
	z=firstzone;
	while(z!=NULL) {
		/* compare coordinates.*/
		if((x>=(z->x1)) && (x<=(z->x2)) &&
			(y>=(z->y1)) && (y<=(z->y2))){
			/* Are flags set? Note! It makes no sense to call this func
			 * with flags value NONE.*/
			if((z->flags & flags) &&
				(z->parent_screen == hr.active_screen)&&
				(z->parent_window == hr.active_screen->active_window))
			  return z;
		}
		z=z->next;
	}
	return NULL;
}


/* Deletes a zone. Returns pointer to freed memory. */
struct hr_zone *hr_delete_zone(struct hr_zone *zone)
{
	struct hr_zone *zz;
	if(zone!=NULL){
		if(zone==hr.firstzone)
		  hr.firstzone=zone->next;
		else{
			zz=hr.firstzone;
			while((zz->next)!=zone)
			  zz=zz->next;
			zz->next=(zz->next->next);
		}
		free(zone);
	}
	return zone;
}


/* Scrolls contents of a zone.*/
void hr_scroll_zone(struct hr_zone *zone,int dx,int dy,int flags)
{
	if(zone!=NULL){
		/* LIS TUKI WRAP_SCROLLILLE!*/
		if(((zone->x1)<(zone->x2))&&((zone->y1)<(zone->y2)))
		  gl_copybox(zone->x1,zone->y1,
						 (zone->x2)-(zone->x1)+1,(zone->y2)-(zone->y1)+1,
						 (zone->x1)+dx,(zone->y1)+dy);
	}
}


/* Clears a zone with some color */
void hr_clear_zone(struct hr_zone *zone,int color,int flags)
{
	if(zone!=NULL)
	  gl_fillbox(zone->x1,zone->y1,
					 (zone->x2)-(zone->x1)+1,
					 (zone->y2)-(zone->y1)+1,color);
}


/* Moves a zone to first place in the list*/
void hr_zone_to_front(struct hr_zone *zp)
{
	struct hr_zone *z;
	if((zp!=NULL)&&(zp!=hr.firstzone)){
		z=hr.firstzone;
		while((z->next)!=zp)
		  z=z->next;
		z->next=(z->next->next);
		z=hr.firstzone;
		hr.firstzone=zp;
		hr.firstzone->next=z;
	}
}


/* Moves a zone to last place in the list */
void hr_zone_to_back(struct hr_zone *zp)
{
	struct hr_zone *z;
	if(zp!=NULL){
		if(zp->next!=NULL){
			z=hr.firstzone;
			while((z->next)!=zp)
			  z=z->next;
			z->next=(z->next->next);
			while((z->next)!=NULL)
			  z=z->next;
			z->next=zp;
			zp->next=NULL;
		}
	}
}
